import sqlite3
from abc import ABC, abstractmethod
from datetime import datetime

from encoo.logger import Logger

import ebank.constants as CONSTANTS
from ebank.utils import sfun


class BaseTrans(ABC):

    _logger = Logger("BaseTrans")

    def __init__(self, file_path, row_start_idx):

        # 流水文件路径
        self.file_path = file_path
        # 开始取数行索引
        self.row_start_idx = row_start_idx

        # 银行名称
        self.bank_name = ""

        # 本方户名
        self.acc_name = ""
        # 本方账号
        self.acc_no = ""

        # 交易日期
        self.trans_date = ""
        # 流水号
        self.flow_no = ""
        # 收入金额
        self.rec_amt = 0.00
        # 支出金额
        self.pay_amt = 0.00
        # 余额
        self.balance = 0.00

        # 对方户名
        self.other_acc_name = ""
        # 对方账号
        self.other_acc_no = ""
        # 对方行名
        self.other_bank_name = ""

        # 用途
        self.purpose = ""
        # 摘要
        self.abstract = ""
        # 备注
        self.remark = ""

        # 回单路径
        self.rct_path = ""
        self.task_id = ""

        self.excel_sheet = sfun.read_xl(self.file_path)
        # 从文件路径中加载信息
        self.__from_filepath()

    def __from_filepath(self):
        """ 从路径中获取信息
        D:\Repos\工商银行\20230604\H_4301016509827101946_20230601-20230604_流水.xlsx
        """
        try:
            fps = self.file_path.split("\\")
            self.bank_name = fps[CONSTANTS.BANK_NAME_IDX]
            self.acc_no = sfun.get_curno(fps[CONSTANTS.BANK_NO_IDX])
            self.acc_name = CONSTANTS.BANK_NO_DICT[self.acc_no]

        except Exception as ex:
            self._logger.error(f"BaseTrans __from_filepath {ex}")

    @abstractmethod
    def build_trans(self, row_idx):
        pass

    def extract(self) -> list[tuple]:
        row_list = []

        try:
            self.task_id = datetime.now().strftime("%Y%m%d%H%M%S")
            for row_idx in range(self.row_start_idx, self.excel_sheet.nrows):
                # 调用子类实现的方法
                self.build_trans(row_idx)

                trans_row = (self.file_path,
                             self.bank_name,
                             self.acc_name,
                             self.acc_no,
                             self.trans_date,
                             self.flow_no,
                             self.rec_amt,
                             self.pay_amt,
                             self.balance,
                             self.other_acc_name,
                             self.other_acc_no,
                             self.other_bank_name,
                             self.purpose,
                             self.abstract,
                             self.remark,
                             self.rct_path,
                             self.task_id,
                             datetime.now(),
                             datetime.now(),
                             1
                             )

                row_list.append(trans_row)
            # end for
        except Exception as ex:
            self._logger.error(f"BaseTrans extract  {ex}")
        # end try
        return row_list

    def to_sqlite(self, db_path=CONSTANTS.SQLITE3_PATH) -> None:
        rows = self.extract()
        self._logger.info(f"BaseTrans to_sqlite {rows}")
        try:
            conn = sqlite3.connect(db_path)
            cur = conn.cursor()
            sql = CONSTANTS.SQL_INSERT_TRANS
            cur.executemany(sql, rows)
            conn.commit()

            cur.close()
            conn.close()
        except Exception as ex:
            self._logger.error(f"BaseTrans to_sqlite {ex}")


class ICBCTrans(BaseTrans):

    '''工商银行
    日期	交易类型	凭证种类	凭证号	对方户名	对方账号	摘要	借方发生额	贷方发生额	余额
    2023-06-01	转账	0	00000000000000000	CHFMCNSH	"1001190709278132949" 	23060135315	0.00 	686,502,400.00 	687,103,056.00
    2023-06-01	转账	0	00000000000000000		"0101000111*********" 	LMS2023053146349	686,502,400.00 	0.00 	600,656.00 '''

    # 流水文件开始取数索引
    ROW_START_IDX = 5

    def __init__(self, file_path, row_start_idx=ROW_START_IDX):
        super().__init__(file_path, row_start_idx)

    def build_trans(self, row_idx):
        sheet = self.excel_sheet
        # 依次按列取值
        self.trans_date = sheet.cell(row_idx, 0).value
        self.flow_no = sheet.cell(row_idx, 3).value
        self.rec_amt = sheet.cell(row_idx, 8).value
        self.pay_amt = sheet.cell(row_idx, 7).value
        self.balance = sheet.cell(row_idx, 9).value
        self.other_acc_name = sheet.cell(row_idx, 4).value
        self.other_acc_no = sheet.cell(row_idx, 5).value
        self.purpose = ""
        self.abstract = sheet.cell(row_idx, 6).value
        return self


class HXBTrans(BaseTrans):

    ''''华夏银行
        序号	交易日期	交易时间	支出金额	收入金额	余额	对方账号	对方户名	对方行名	核心流水号	交易描述	摘要	凭证号码
        1	2023-02-13	11:28:34	7.00		469.45	60220903	网上银行结算手续费收入		475384	网银结算手续费		
        2	2023-02-13	11:28:34	63,056.41		476.45	99060171480000008	同业系统往来-无锡分行	江苏银行股份有限公司	475384	网上银行跨行付款	理财利息'''

    # 流水文件开始取数索引
    ROW_START_IDX = 7

    def __init__(self, file_path, row_start_idx=ROW_START_IDX):
        super().__init__(file_path, row_start_idx)

    def build_trans(self, row_idx):
        sheet = self.excel_sheet
        # 依次按列取值
        self.trans_date = sheet.cell(row_idx, 1).value
        self.flow_no = sheet.cell(row_idx, 9).value
        self.rec_amt = sheet.cell(row_idx, 4).value
        self.pay_amt = sheet.cell(row_idx, 3).value
        self.balance = sheet.cell(row_idx, 5).value
        self.other_acc_name = sheet.cell(row_idx, 7).value
        self.other_acc_no = sheet.cell(row_idx, 6).value
        self.other_bank_name = sheet.cell(row_idx, 8).value
        self.purpose = ""
        self.abstract = sheet.cell(row_idx, 11).value
        return self


if (__name__ == "__main__"):
    bfr = HXBTrans(
        r"D:\Repos\华夏银行\20230606\H_12592000000000515_20230201-20230228_流水.xls").to_sqlite()




import base64
import os
import re
import urllib

import xlrd
import xlwt
from dateutil.parser import parser
from pypdf import PdfReader, PdfWriter


def fmt_amt(amt_str="人民币（大写）： 肆仟万元整 ￥ 40,000,000.00元"):
    regex = r"([\d|,|.])+"
    matches = re.finditer(regex, str(amt_str))
    match_text = ""
    for matchNum, match in enumerate(matches, start=1):
        match_text = match.group()
    return match_text.replace(",", "")


def fmt_trans_date(trans_date="2022-09-23 11:41:30"):
    try:
        if (isinstance(trans_date, float)):
            return xlrd.xldate_as_datetime(
                trans_date, 0).strftime("%Y-%m-%d")
        else:
            new_date = trans_date.replace(
                "年", "-").replace("月", "-").replace("日", " ").replace("时", ":").replace("分", ":").replace("秒", "")
        return parser().parse(new_date).strftime("%Y-%m-%d")
    except:
        return trans_date


def get_bankname(file_path: str, idx: int):
    fps = file_path.split("\\")
    return fps[idx]


def get_curno(file_name):
    regex = r"(?<=H_)\d*"
    finds = re.findall(regex, file_name)
    if len(finds) > 0:
        return finds[0]
    else:
        return ""


def read_xl(xl_path, sheet_idx=0):
    xls = xlrd.open_workbook(xl_path, formatting_info=False)
    sheet = xls.sheet_by_index(sheet_idx)
    return sheet


def wt_rct_excel(xl_path, rct_list) -> str:

    excel_header = ["回单流水号", "我方公司名称", "银行名称", "本方银行账号", "日期",
                    "收入金额", "支出金额", "余额", "对方户名", "对方账号", "用途", "摘要", "回单路径"]

    xl = xlwt.Workbook(encoding="uft-8")
    sheet = xl.add_sheet("sheet1", cell_overwrite_ok=False)

    col_idx = 0
    for col_name in excel_header:
        sheet.write(0, col_idx, col_name)
        col_idx = col_idx+1

    row_idx = 1
    for rct in rct_list:
        sheet.write(row_idx, 0, rct.flow_no)  # 流水号
        sheet.write(row_idx, 1, rct.company_name)
        sheet.write(row_idx, 2, rct.bank_name)
        sheet.write(row_idx, 3, rct.cur_no)
        sheet.write(row_idx, 4, rct.trans_date)
        sheet.write(row_idx, 5, rct.rec_amt)
        sheet.write(row_idx, 6, rct.pay_amt)
        sheet.write(row_idx, 8, rct.ant_name)
        sheet.write(row_idx, 9, rct.ant_no)
        sheet.write(row_idx, 12, rct.file_path)

        row_idx = row_idx+1

    xl.save(xl_path)
    return xl_path


def wt_bankflow_excel(xl_path, bankflow_list) -> str:

    excel_header = ["公司名称", "银行名称", "本方银行账号", "日期", "流水号",
                    "收入金额", "支出金额", "余额", "对方户名", "对方账号", "用途",
                    "摘要", "电子回单", "服务器回单路径", "SrvUUID"]

    xl = xlwt.Workbook(encoding="uft-8")
    sheet = xl.add_sheet("sheet1", cell_overwrite_ok=False)

    col_idx = 0
    for col_name in excel_header:
        sheet.write(0, col_idx, col_name)
        col_idx = col_idx+1

    row_idx = 1
    for bf in bankflow_list:
        sheet.write(row_idx, 0, bf.company_name)
        sheet.write(row_idx, 1, bf.bank_name)
        sheet.write(row_idx, 2, bf.bank_no)
        sheet.write(row_idx, 3, bf.trans_date)
        sheet.write(row_idx, 4, bf.flow_no)
        sheet.write(row_idx, 5, bf.rec_amt)
        sheet.write(row_idx, 6, bf.pay_amt)
        sheet.write(row_idx, 7, bf.balance)
        sheet.write(row_idx, 8, bf.ant_name)
        sheet.write(row_idx, 9, bf.ant_no)
        sheet.write(row_idx, 10, bf.purpose)
        sheet.write(row_idx, 11, bf.summary)

        row_idx = row_idx+1

    xl.save(xl_path)
    return xl_path


def split_pdf(pdf_file, output_dir=None):
    f_name = os.path.basename(pdf_file)
    if output_dir is None:
        output_dir = os.path.join(os.path.dirname(pdf_file), "subpdf")

    pdfs = []
    try:
        if os.path.exists(output_dir) == False:
            os.mkdir(output_dir)
        reader = PdfReader(pdf_file)
        for index in range(len(reader.pages)):
            writer = PdfWriter()
            pageObj = reader.pages[index]
            writer.add_page(pageObj)
            newpath = os.path.join(output_dir, f"{index}_{f_name}")
            with open(newpath, 'wb') as fw:
                writer.write(fw)

            pdfs.append(newpath)
    except Exception as e:
        print(e)

    return pdfs


def get_file_content_as_base64(path, urlencoded=False):
    with open(path, "rb") as f:
        content = base64.b64encode(f.read()).decode("utf8")
        if urlencoded:
            content = urllib.parse.quote_plus(content)
    return content


if __name__ == "__main__":
    kv = {"k": "v"}
    print(kv.get("k"))
    print(kv.get("kss", "ss"))



import base64
import json

import requests
from encoo.logger import Logger

import ebank.utils.sfun as util


class BaiduOCR:

    __logger = Logger("BaiduOCR")

    def __init__(self) -> None:
        pass

    def ocr_table(self, file_path):

        url = "https://aip.baidubce.com/rest/2.0/ocr/v1/table?access_token=" + \
            self.get_access_token()

        payload = f"pdf_file={util.get_file_content_as_base64(file_path, True)}"
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Accept': 'application/json'
        }
        ocr_tables = []

        try:
            response = requests.request(
                "POST", url, headers=headers, data=payload)

            self.__logger.debug(f"BaiduOCR response {response.content}")
            ocr_tables = self.convert_resp(response.content)
        except Exception as ex:
            self.__logger.error(f"BaiduOCR {ex}")

        return ocr_tables

    def convert_resp(self, response) -> list[dict]:
        resp_json = json.loads(response)
        ocr_tables = []
        for t in resp_json["tables_result"]:
            ocr_t = {}
            for body in t["body"]:
                k = f"{body['col_start']}-{body['row_start']}-{body['col_end']}-{body['row_end']}"
                v = body['words']
                ocr_t[k] = v
            # end for
            ocr_tables.append(ocr_t)
        self.__logger.debug(f"BaiduOCR ocr_tables {ocr_tables}")

        return ocr_tables

    def get_access_token(self):
        API_KEY = "WYz5wiwt1PoWQkBqlG2EIGIG-"
        SECRET_KEY = "chYQKg2vkn9eTdB5h80GgKSzjwCnmxat"

        url = "https://aip.baidubce.com/oauth/2.0/token"
        params = {"grant_type": "client_credentials",
                  "client_id": API_KEY, "client_secret": SECRET_KEY}
        return str(requests.post(url, params=params).json().get("access_token"))


if (__name__ == "__main__"):
    BaiduOCR().ocr_table(
        r"D:\Ouye\12家银行流水回单\浙商银行\欧冶商业保理有限责任公司\20221107\H 2900000000121800049625 20221101~20221107.pdf")



import re
from encoo.logger import Logger
import pdfplumber
from utils import baiduocr, misc as util
from utils.filesearch import FileSearch


class ReceiptFactory:
    """网银回单解析
    昆仑银行，汉口银行，浦发银行，进出口银行，民生银行，上海银行，交通银行，华夏银行，安徽农金，北京银行，浙商银行
    Returns:
        回单数据保存成xls文件
    """
    RECEIPT_CLS = {
        "昆仑银行": "KunLunRct", "汉口银行": "HanKouRct", "浦发银行": "PuFaRct",
        "进出口银行": "JinChuKouRct", "民生银行": "MinShengRct", "上海银行": "ShangHaiRct",
        "交通银行": "JiaoTongRct", "华夏银行": "HuaXiaRct", "安徽农金": "AnHuiNongJinRct",
        "北京银行": "BeiJingRct", "浙商银行": "ZheShangRct"}

    ROOT_DIR = r"C:\nas-个人文件夹\RPA用（误删）\RPA银行下载文件"
    __logger = Logger("ReceiptFactory")

    def __init__(self, run_date, root_dir=ROOT_DIR) -> None:

        self.__file_list = FileSearch(
            run_date, root_dir).search_rct_files()

    def extract_to_xls(self, xlsPath: str):
        try:
            row_list = self.get_rct_data()
            util.wt_rct_excel(xlsPath, row_list)
        except Exception as e:
            self.__logger.error(f"ReceiptFactory extract_to_xls-{e}")

        return xlsPath

    def get_rct_data(self):

        row_list = []
        for f in self.__file_list:
            rcts = self.__extract_pdf(f)

            for rct in rcts:
                row_list.append(rct)

        return row_list

    def __extract_pdf(self, pdf_path):
        rcts = []
        bank_name = pdf_path.split("\\")[-4]

        rct_cls = self.RECEIPT_CLS.get(bank_name)
        if rct_cls is not None:
            cur_cls = globals()[rct_cls]
            rcts = cur_cls(file_path=pdf_path).extract()
        else:
            self.__logger.info(f"__extract_pdf not find cls for {bank_name}")

        return rcts


class Receipt:

    _logger = Logger("Receipt")

    def __init__(self, file_path=None, *args):
        self.file_path = file_path
        self.pay_no = ""
        self.pay_name = ""
        self.rec_no = ""
        self.rec_name = ""

        self.amt = 0
        self.pay_amt = 0
        self.rec_amt = 0

        self.trans_date = ""

        self.cur_no = ""
        self.cur_name = ""
        self.ant_no = ""
        self.ant_name = ""

        self.company_name = ""
        self.bank_name = ""

        self.rct_no = ""
        self.flow_no = ""

        self.pdf_tables = []
        self.is_multi_pages = False
        self.sub_pdfs = []

        self.__load()
        self.__init_data(args)

    def __load(self):

        if self.file_path is not None:
            pdf = pdfplumber.open(self.file_path)
            if len(pdf.pages) > 1:
                self.is_multi_pages = True
                self.sub_pdfs = util.split_pdf(self.file_path)
            else:
                self.pdf_tables = pdf.pages[0].extract_tables()
                self.pdf_text = pdf.pages[0].extract_text()

    def __init_data(self, *args):
        if len(args[0]) == 3:
            self.company_name = args[0][0]
            self.bank_name = args[0][1]
            self.cur_no = args[0][2]
        elif self.file_path is not None:
            file_paths = self.file_path.split("\\")
            self.company_name = file_paths[-3]
            self.bank_name = file_paths[-4]
            self.cur_no = util.get_curno(file_paths[-1])
        else:
            pass  # 不初始化值

    def fmt(self):
        if self.pay_name is not None:
            self.pay_name = self.pay_name.replace("\n", "")
        if self.rec_name is not None:
            self.rec_name = self.rec_name.replace("\n", "")
        # cal pay / rec amt
        if (self.pay_no == self.cur_no):
            self.pay_amt = self.amt
            self.cur_name = self.pay_name

            self.ant_no = self.rec_no
            self.ant_name = self.rec_name

        if (self.rec_no == self.cur_no):
            self.rec_amt = self.amt
            self.cur_name = self.rec_name

            self.ant_no = self.pay_no
            self.ant_name = self.pay_name
        return self

    def extract(self) -> list["Receipt"]:
        pass


class KunLunRct(Receipt):

    def __init__(self, file_path=""):
        super().__init__(file_path)

    def extract(self) -> list[Receipt]:
        self._logger.debug(f"KunLunRct  {self.pdf_tables}")

        rct_list = []
        try:
            for table in self.pdf_tables:
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                rct.pay_name = table[0][2]
                rct.rec_name = table[0][-1]

                rct.pay_no = table[1][2]
                rct.rec_no = table[1][-1]

                rct.amt = util.fmt_amt(table[5][1])
                rct.trans_date = table[9][-1]
                rct.flow_no = table[9][-4]

                rct.fmt()
                rct_list.append(rct)
        except Exception as ex:
            self._logger.debug(f"KunLunRct  {ex}")

        return rct_list


class HanKouRct(Receipt):

    def __init__(self, file_path=""):
        super().__init__(file_path)

    def extract(self) -> list[Receipt]:
        self._logger.debug(f"HanKouRct  {self.pdf_tables}")

        rct_list = []
        try:
            for table in self.pdf_tables:
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                # rct.rct_no = ""
                rct.pay_name = table[1][2]
                rct.rec_name = table[1][-3]

                rct.pay_no = table[2][2]
                rct.rec_no = table[2][-3]

                rct.amt = util.fmt_amt(table[4][-3])
                rct.trans_date = util.fmt_trans_date(table[0][-3])
                rct.flow_no = table[0][2]

                rct.fmt()
                rct_list.append(rct)
        except Exception as ex:
            self._logger.debug(f"HanKouRct  {ex}")

        return rct_list


class PuFaRct(Receipt):

    def __init__(self, file_path=""):
        super().__init__(file_path)

    def extract(self) -> list[Receipt]:
        self._logger.debug(f"PuFaRct  {self.pdf_tables}")

        rct_list = []
        try:
            for table in self.pdf_tables:
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                rct.rct_no = table[0][2]
                rct.pay_name = table[1][2]
                rct.rec_name = table[1][-2]

                rct.pay_no = table[2][2]
                rct.rec_no = table[2][-2]

                rct.amt = util.fmt_amt(table[6][-4])
                rct.trans_date = util.fmt_trans_date(table[5][2])
                rct.flow_no = table[5][-1]

                rct.fmt()
                rct_list.append(rct)
        except Exception as ex:
            self._logger.debug(f"PuFaRct {ex}")

        return rct_list


class JinChuKouRct(Receipt):
    '''使用baidu OCR '''

    KEY_RCT_NO = ""
    KEY_PAY_NAME = ""
    KEY_PAY_NO = ""
    KEY_REC_NAME = "8-1-9-2"
    KEY_REC_NO = "8-2-9-3"

    KEY_AMT = "7-4-9-5"
    KEY_DATE = "0-0-5-1"
    KEY_FLOW_NO = "5-0-9-1"

    def __init__(self, file_path=None, *args):
        super().__init__(file_path, *args)

    rct_list = []

    def extract(self) -> list[Receipt]:
        if self.is_multi_pages == True:
            for pdf_path in self.sub_pdfs:
                rcts = JinChuKouRct(pdf_path, self.company_name,
                                    self.bank_name, self.cur_no).extract()
                for rct in rcts:
                    self.rct_list.append(rct)
        else:
            return self.__extract(self.pdf_text)
        return self.rct_list

    def __extract(self, pdf_text):
        self._logger.debug(f"JinChuKouRct pdf_text{pdf_text}")

        rcts = []
        try:
            pdf_kvs = []
            if (len(self.pdf_tables) > 0):
                pdf_kvs = baiduocr().ocr_table(self.file_path)
            else:
                self._logger.error(f"JinChuKouRct not table {self.file_path}")
            self._logger.debug(
                f"JinChuKouRct {self.file_path} pdf_kv{pdf_kvs}")

            for kv in pdf_kvs:
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                rct.pay_name = kv.get(self.KEY_PAY_NAME, "")
                rct.pay_no = kv.get(self.KEY_PAY_NO, "")
                rct.rec_name = kv.get(self.KEY_REC_NAME, "")
                rct.rec_no = kv.get(self.KEY_REC_NO, "")

                rct.amt = util.fmt_amt(kv.get(self.KEY_AMT, ""))

                date_str = s = re.findall(
                    r"\d*", kv.get(self.KEY_DATE, ""))[0]
                rct.trans_date = util.fmt_trans_date(date_str)

                rct.flow_no = kv.get(self.KEY_FLOW_NO, "")

                rct.fmt()
                rcts.append(rct)
        except Exception as ex:
            self._logger.error(f"JinChuKouRct __extract {ex}")

        return rcts


class MinShengRct(Receipt):

    def __init__(self, file_path=""):
        super().__init__(file_path)

    def extract(self) -> list[Receipt]:
        self._logger.debug(f"MinShengRct  {self.pdf_tables}")

        rct_list = []
        try:
            for table in self.pdf_tables:
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                # rct.rct_no = ""
                rct.pay_name = table[2][2]
                rct.rec_name = table[2][-1]

                rct.pay_no = table[3][2]
                rct.rec_no = table[3][-1]

                rct.amt = util.fmt_amt(table[5][1])
                rct.trans_date = util.fmt_trans_date(table[6][-2])
                rct.flow_no = table[7][1]

                rct.fmt()
                rct_list.append(rct)
        except Exception as ex:
            self._logger.debug(f"MinShengRct  {ex}")

        return rct_list


class ZheShangRct(Receipt):

    def __init__(self, file_path=None, *args):
        super().__init__(file_path, *args)

    rct_list = []

    def extract(self) -> list[Receipt]:
        if self.is_multi_pages == True:
            for pdf_path in self.sub_pdfs:
                rcts = ZheShangRct(pdf_path, self.company_name,
                                   self.bank_name, self.cur_no).extract()
                for rct in rcts:
                    self.rct_list.append(rct)
        else:
            return self.__extract()
        return self.rct_list

    def __extract(self):
        self._logger.debug(f"ZheShangRct  {self.pdf_tables}")

        rct_list = []
        try:
            for table in self.pdf_tables:
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                # rct.rct_no = ""
                rct.pay_name = str(table[0][1]).removeprefix("户名:")
                rct.rec_name = str(table[0][-1]).removeprefix("户名:")

                rct.pay_no = str(table[1][1]).removeprefix("账号:")
                rct.rec_no = str(table[1][-1]).removeprefix("账号:")

                rct.amt = util.fmt_amt(table[3][2])

                trans = table[4][0]
                flow_nos = re.findall(r"(?<=交易流水号：)\d*", trans)
                if (len(flow_nos) > 0):
                    rct.flow_no = flow_nos[0]

                trans_dates = re.findall(r"(?<=交易时间：)\d{4}-\d{2}-\d{2}", trans)
                if (len(trans_dates) > 0):
                    rct.trans_date = trans_dates[0]

                rct.fmt()
                rct_list.append(rct)
        except Exception as ex:
            self._logger.debug(f"ZheShangRct  {ex}")

        return rct_list


class BeiJingRct(Receipt):

    def __init__(self, file_path=None, *args):
        super().__init__(file_path, *args)

    rct_list = []

    def extract(self) -> list[Receipt]:
        if self.is_multi_pages == True:
            for pdf_path in self.sub_pdfs:
                rcts = BeiJingRct(pdf_path, self.company_name,
                                  self.bank_name, self.cur_no).extract()
                for rct in rcts:
                    self.rct_list.append(rct)
        else:
            return self.__extract()
        return self.rct_list

    def __extract(self):
        self._logger.debug(f"BeiJingRct  {self.pdf_tables}")

        rct_list = []
        try:
            for table in self.pdf_tables:
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                # rct.rct_no = ""
                rct.pay_name = table[0][1]
                rct.rec_name = table[0][-1]

                rct.pay_no = table[1][1]
                rct.rec_no = table[1][-1]

                rct.amt = util.fmt_amt(table[4][1])
                rct.trans_date = util.fmt_trans_date(table[3][-1])

                rct.flow_no = str(table[7][-1])[0:19]

                rct.fmt()
                rct_list.append(rct)
        except Exception as ex:
            self._logger.debug(f"BeiJingRct  {ex}")

        return rct_list


class ShangHaiRct(Receipt):

    RE_SPLIT_GROUP = r"(?=记账日期)[\S\s]*?(?=打印时间)"
    RE_RCT_NO = r"(?<=回单编号:)[A-Z|\d]*"
    RE_PAY_NAME = r"(?<=付款人名称:)\S*"
    RE_PAY_NO = r"(?<=付款人账号:)\S*"
    RE_AMT = r"(?<=金额\(小写\):)\S*"
    RE_DATE = r"(?<=记账日期:)[\d*年\d*月\d*日]*"

    def __init__(self, file_path=None, *args):
        super().__init__(file_path, *args)

    rct_list = []

    def extract(self) -> list[Receipt]:
        if self.is_multi_pages == True:
            for pdf_path in self.sub_pdfs:
                rcts = ShangHaiRct(pdf_path, self.company_name,
                                   self.bank_name, self.cur_no).extract()
                for rct in rcts:
                    self.rct_list.append(rct)
        else:
            return self.__extract(self.pdf_text)
        return self.rct_list

    def __extract(self, pdf_text):
        self._logger.debug(f"ShangHaiRct __extract{pdf_text}")
        rcts = []
        try:
            matches = re.finditer(
                self.RE_SPLIT_GROUP, pdf_text, re.MULTILINE)
            for match in matches:
                group_text = match.group().replace(" ", "")
                # print(f"group_text-{group_text}")
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                iter_rct_no = re.finditer(self.RE_RCT_NO, group_text)
                rct.rct_no = next(iter_rct_no).group()

                iter_pay_name = re.finditer(self.RE_PAY_NAME, group_text)
                pay_name_group = next(
                    iter_pay_name).group().split("收款人名称:")
                rct.pay_name = pay_name_group[0]
                rct.rec_name = pay_name_group[1]

                iter_pay_no = re.finditer(self.RE_PAY_NO, group_text)
                pay_no_group = next(iter_pay_no).group().split("收款人账号:")
                rct.pay_no = pay_no_group[0]
                rct.rec_no = pay_no_group[1]

                iter_amt = re.finditer(self.RE_AMT, group_text)
                rct.amt = util.fmt_amt(next(iter_amt).group())

                iter_date = re.finditer(self.RE_DATE, group_text)
                rct.trans_date = util.fmt_trans_date(next(iter_date).group())
                # rct.flow_no=""

                rct.fmt()
                rcts.append(rct)
        except Exception as ex:
            self._logger.error(f"ShangHaiRct __extract {ex}")

        return rcts


class JiaoTongRct(Receipt):

    RE_SPLIT_GROUP = r"(?=回单编号)[\S\s]*?(?=批次号)"
    RE_RCT_NO = r"(?<=回单编号：)[A-Z|\d]*"
    RE_PAY_NAME = r"付款人名称：(.*?)开户行名称"
    RE_RCE_NAME = r"收款人名称：(.*?)开户行名称"

    RE_PAY_NO = r"(?<=付款人账号：)\d*"
    RE_RCE_NO = r"(?<=收款人账号：)\d*"

    RE_AMT = r"(?<=CNY金额：)[.|,|\d]*"
    RE_DATE = r"(?<=记账日期：)\d*"
    RE_FLOW_NO = r"(?<=会计流水号：)[A-Z|\d]*"

    def __init__(self, file_path=None, *args):
        super().__init__(file_path, *args)

    rct_list = []

    def extract(self) -> list[Receipt]:
        if self.is_multi_pages == True:
            for pdf_path in self.sub_pdfs:
                rcts = JiaoTongRct(pdf_path, self.company_name,
                                   self.bank_name, self.cur_no).extract()
                for rct in rcts:
                    self.rct_list.append(rct)
        else:
            return self.__extract(self.pdf_text)
        return self.rct_list

    def __extract(self, pdf_text):
        self._logger.debug(f"JiaoTongRct __extract{pdf_text}")
        rcts = []
        try:
            matches = re.finditer(
                self.RE_SPLIT_GROUP, pdf_text, re.MULTILINE)
            for match in matches:
                group_text = match.group().replace(" ", "").replace("\n", "")
                print(f"group_text-{group_text}")
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                iter_rct_no = re.finditer(self.RE_RCT_NO, group_text)
                rct.rct_no = next(iter_rct_no).group()

                rct.pay_name = re.findall(self.RE_PAY_NAME, group_text)[0]
                rct.rec_name = re.findall(self.RE_RCE_NAME, group_text)[0]

                rct.pay_no = re.findall(self.RE_PAY_NO, group_text)[0]
                rct.rec_no = re.findall(self.RE_RCE_NO, group_text)[0]

                iter_amt = re.finditer(self.RE_AMT, group_text)
                rct.amt = util.fmt_amt(next(iter_amt).group())

                iter_date = re.finditer(self.RE_DATE, group_text)
                rct.trans_date = util.fmt_trans_date(next(iter_date).group())

                iter_flow_no = re.finditer(self.RE_FLOW_NO, group_text)
                rct.flow_no = util.fmt_trans_date(next(iter_flow_no).group())

                rct.fmt()
                rcts.append(rct)
        except Exception as ex:
            self._logger.error(f"JiaoTongRct __extract {ex}")

        return rcts


class HuaXiaRct(Receipt):

    # RE_SPLIT_GROUP = r"(?=回单编号)[\S\s]*?(?=批次号)"
    RE_RCT_NO = r"(?<=回单编码:)[A-Z|\d]*"
    RE_PAY_NAME = r"转出户名:(.*?)转入户名"
    RE_RCE_NAME = r"(?<=转入户名:)\S*"

    RE_PAY_NO = r"(?<=转出账号:)\d*"
    RE_RCE_NO = r"(?<=转入账号:)\d*"

    RE_AMT = r"(?<=金额:)[.|,|\d]*"
    RE_DATE = r"(?<=交易日期)\S*"
    RE_FLOW_NO = r"(?<=流水号:)[A-Z|\d]*"

    def __init__(self, file_path=None, *args):
        super().__init__(file_path, *args)

    rct_list = []

    def extract(self) -> list[Receipt]:
        if self.is_multi_pages == True:
            for pdf_path in self.sub_pdfs:
                rcts = HuaXiaRct(pdf_path, self.company_name,
                                 self.bank_name, self.cur_no).extract()
                for rct in rcts:
                    self.rct_list.append(rct)
        else:
            return self.__extract(self.pdf_text)
        return self.rct_list

    def __extract(self, pdf_text):

        pdf_text = pdf_text.replace(" ", "")
        self._logger.debug(f"HuaXiaRct __extract{pdf_text}")
        rcts = []
        try:
            if (pdf_text != ""):
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                iter_rct_no = re.finditer(self.RE_RCT_NO, pdf_text)
                rct.rct_no = next(iter_rct_no).group()

                if ("转账业务" in pdf_text):
                    rct.pay_name = re.findall(self.RE_PAY_NAME, pdf_text)[0]
                    rct.rec_name = re.findall(self.RE_RCE_NAME, pdf_text)[0]

                    rct.pay_no = re.findall(self.RE_PAY_NO, pdf_text)[0]
                    rct.rec_no = re.findall(self.RE_RCE_NO, pdf_text)[0]

                    iter_amt = re.finditer(self.RE_AMT, pdf_text)
                    rct.amt = util.fmt_amt(next(iter_amt).group())

                    iter_date = re.finditer(self.RE_DATE, pdf_text)
                    rct.trans_date = util.fmt_trans_date(
                        next(iter_date).group())
                # end if
                iter_flow_no = re.finditer(self.RE_FLOW_NO, pdf_text)
                rct.flow_no = next(iter_flow_no).group()

                rct.fmt()
                rcts.append(rct)
        except Exception as ex:
            self._logger.error(f"HuaXiaRct __extract {ex}")

        return rcts


class AnHuiNongJinRct(Receipt):
    '''使用baidu OCR '''

    KEY_RCT_NO = ""
    KEY_PAY_NAME = "2-0-3-1"
    KEY_PAY_NO = "2-1-3-2"
    KEY_REC_NAME = "5-0-6-1"
    KEY_REC_NO = "5-1-6-2"

    KEY_AMT = "5-3-6-4"
    KEY_DATE = "5-5-6-6"
    KEY_FLOW_NO = "5-6-6-7"

    def __init__(self, file_path=None, *args):
        super().__init__(file_path, *args)

    rct_list = []

    def extract(self) -> list[Receipt]:
        if self.is_multi_pages == True:
            for pdf_path in self.sub_pdfs:
                rcts = AnHuiNongJinRct(pdf_path, self.company_name,
                                       self.bank_name, self.cur_no).extract()
                for rct in rcts:
                    self.rct_list.append(rct)
        else:
            return self.__extract(self.pdf_text)
        return self.rct_list

    def __extract(self, pdf_text):
        rcts = []
        try:
            pdf_kvs = baiduocr().ocr_table(self.file_path)
            self._logger.debug(
                f"AnHuiNongJinRct {self.file_path} pdf_kv{pdf_kvs}")

            for kv in pdf_kvs:
                rct = Receipt()
                rct.file_path = self.file_path
                rct.company_name = self.company_name
                rct.bank_name = self.bank_name
                rct.cur_no = self.cur_no

                rct.pay_name = kv.get(self.KEY_PAY_NAME, "")
                rct.pay_no = kv.get(self.KEY_PAY_NO, "")
                rct.rec_name = kv.get(self.KEY_REC_NAME, "")
                rct.rec_no = kv.get(self.KEY_REC_NO, "")

                rct.amt = util.fmt_amt(kv.get(self.KEY_AMT, ""))

                rct.trans_date = util.fmt_trans_date(
                    kv.get(self.KEY_DATE, ""))
                rct.flow_no = kv.get(self.KEY_FLOW_NO, "")

                rct.fmt()
                rcts.append(rct)
        except Exception as ex:
            self._logger.error(f"AnHuiNongJinRct __extract {ex}")

        return rcts


if __name__ == "__main__":
    pass



import os


class FileSearch(object):

    BANK_LIST = ["北京银行", "渤海银行", "大连银行", "广发银行", "汉口银行", "昆仑银行",
                 "马农商&合肥科技农商行", "农商银行", "浙商银行", "邮储银行", "进出口银行", "华夏银行",]

    XLS_SUFFIX = [".xls", ".csv", ".xlsx"]
    PDF_SUFFIX = [".pdf"]
    FILE_PREFIX = "H"

    ROOT_DIR = r"C:\nas-个人文件夹\RPA用（误删）\RPA银行下载文件"
    __sub_dirs = []

    def __init__(self,
                 run_date: str,
                 root_dir=ROOT_DIR) -> None:

        self.run_date = run_date
        self.root_dir = root_dir

    def search_bankflow_files(self) -> list[str]:
        return self.search_files(file_suffix=self.XLS_SUFFIX,
                                 bank_list=self.BANK_LIST)

    def search_rct_files(self) -> list[str]:
        return self.search_files(file_suffix=self.PDF_SUFFIX)

    def search_files(self, file_suffix=None, bank_list=None) -> list[str]:
        found_files = []
        dirs = self.search_dirs(bank_list)
        for dir in dirs:
            for f in os.listdir(dir):
                f_path = os.path.join(dir, f)
                if os.path.basename(f_path).startswith(self.FILE_PREFIX):
                    if file_suffix is None:
                        found_files.append(f_path)
                    elif os.path.splitext(f)[-1] in file_suffix:
                        found_files.append(f_path)
                        print(f"match:{f_path}")
                else:
                    print(f"file_prefix not match:{f_path}")
        return found_files

    def search_dirs(self, bank_list: list[str] = None):
        self.__traversal_dirs(self.root_dir)

        match_dirs = []
        for dir in self.__sub_dirs:
            if os.path.basename(dir) == self.run_date:
                if bank_list is None:
                    match_dirs.append(dir)
                elif (dir.split("\\")[-3]) in bank_list:
                    match_dirs.append(dir)
        return match_dirs

    def __traversal_dirs(self, path):
        for item in os.scandir(path):
            if item.is_dir():
                self.__sub_dirs.append(item.path)
                self.__traversal_dirs(item)
            else:
                pass


if __name__ == "__main__":
    FileSearch("2022-11-07", r"D:\Ouye\12家银行流水回单").search_rct_files()



import sqlite3

from encoo.logger import Logger

from ebank import constants as CONSTANTS
from ebank.utils import sfun as util
from ebank.utils.filesearch import FileSearch


class Factory:

    __logger = Logger("ebank factory")

    """（根目录）\(银行名称)\(日期)\[H|T]_网银账号_yyyyMMdd-yyyyMMdd[.xls|.pdf|.xlsx]
    """

    def __init__(self, trans_file):
        self.trans_file = trans_file
        self.bank_name = util.get_bankname(trans_file, CONSTANTS.BANK_NAME_IDX)

    def to_sqlite(self):
        '''判断该银行存在处理类'''
        if self.bank_name in CONSTANTS.BANK_TRANS_CLS.keys():
            class_name = CONSTANTS.BANK_TRANS_CLS.get(self.bank_name)
            module_meta = __import__(
                "ebank.trans", globals(), locals(), f"[{class_name}]")
            class_meta = getattr(module_meta, class_name)

            # 调用具体处理类
            class_meta(file_path=self.trans_file).to_sqlite()
        else:
            self.__logger.error(f"to_sqlite no cls for {self.bank_name}")

    @staticmethod
    def query_trans_bydate(start_date, end_date):
        try:
            conn = sqlite3.connect(CONSTANTS.SQLITE3_PATH)
            cur = conn.cursor()
            sql = CONSTANTS.SQL_QUERY_TRANS_BYDATE.format(start_date, end_date)
            cur.execute(sql)
            result = cur.fetchall()
            print(result)
            conn.commit()

            cur.close()
            conn.close()
        except Exception as ex:
            print(ex)


if __name__ == "__main__":
    Factory.query_trans_bydate("2023-02-10", "2023-02-10")



#!/usr/bin/env python
# coding:utf-8

from hashlib import md5

import requests

import ebank.utils.sfun as util


class Chaojiying_Client(object):

    def __init__(self, username, password, soft_id):
        self.username = username
        password = password.encode('utf8')
        self.password = md5(password).hexdigest()
        self.soft_id = soft_id
        self.base_params = {
            'user': self.username,
            'pass2': self.password,
            'softid': self.soft_id,
        }
        self.headers = {
            'Connection': 'Keep-Alive',
            'User-Agent': 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)',
        }

    def PostPic_base64(self, file_path, codetype):
        """
        im: 图片字节
        codetype: 题目类型 参考 http://www.chaojiying.com/price.html
        """
        base64_str = util.get_file_content_as_base64(file_path, False)
        params = {
            'codetype': codetype,
            'file_base64': base64_str
        }
        params.update(self.base_params)
        r = requests.post(
            'http://upload.chaojiying.net/Upload/Processing.php', data=params, headers=self.headers)
        return r.json()


if __name__ == '__main__':
    chaojiying = Chaojiying_Client('15021869005', 'huangrui', '941459')
    imrslt = chaojiying.PostPic_base64(
        r"C:\Users\renjunfeng\Downloads\Chaojiying_Python\chaojiying_Python\a.jpg", 1902)
    print(imrslt.get("pic_str", ""))



import sqlite3
from abc import ABC, abstractmethod
from datetime import datetime

from encoo.logger import Logger

import ebank.constants as CONSTANTS
from ebank.utils import sfun


class BaseBalance(ABC):

    _logger = Logger("BaseBalance")

    def __init__(self, file_path, row_start_idx):

        # 文件路径
        self.file_path = file_path
        # 开始取数行索引
        self.row_start_idx = row_start_idx

        # 银行名称
        self.bank_name = ""
        # 本方户名
        self.acc_name = ""
        # 账号
        self.acc_no = ""

        # 查询时间
        self.query_date = ""
        # 可用余额
        self.balance = ""

        # 币种
        self.currency = ""
        self.task_id = ""

        self.excel_sheet = sfun.read_xl(self.file_path)
        # 从文件路径中加载信息
        self.__from_filepath()

    def __from_filepath(self):
        """ 从路径中获取信息
        "D:\Repos\工商银行\20230604\工行江苏省分行1_20230605_余额.xlsx"
        """
        try:
            fps = self.file_path.split("\\")
            self.bank_name = fps[CONSTANTS.BANK_NAME_IDX]

        except Exception as ex:
            self._logger.error(f"BaseBalance __from_filepath {ex}")

    @abstractmethod
    def build_self(self, row_idx):
        pass

    def extract(self) -> list[tuple]:
        row_list = []

        try:
            self.task_id = datetime.now().strftime("%Y%m%d%H%M%S")
            for row_idx in range(self.row_start_idx, self.excel_sheet.nrows):
                # 调用子类实现的方法
                self.build_self(row_idx)

                trans_row = (self.file_path,
                             self.bank_name,
                             self.acc_name,
                             self.acc_no,
                             self.balance,
                             self.query_date,
                             self.currency,
                             self.task_id,
                             datetime.now(),
                             datetime.now(),
                             1)

                row_list.append(trans_row)
            # end for
        except Exception as ex:
            self._logger.error(f"BaseTrans extract  {ex}")
        # end try
        return row_list

    def to_sqlite(self, db_path=CONSTANTS.SQLITE3_PATH) -> None:
        rows = self.extract()
        self._logger.info(f"BaseBalance to_sqlite {rows}")
        try:
            conn = sqlite3.connect(db_path)
            cur = conn.cursor()
            sql = CONSTANTS.SQL_INSERT_BALANCE
            cur.executemany(sql, rows)
            conn.commit()

            cur.close()
            conn.close()
        except Exception as ex:
            self._logger.error(f"BaseBalance to_sqlite {ex}")


class ICBCBalance(BaseBalance):

    ''' 工商银行
        账　号	币种	钞汇标志	账户属性	昨日余额	当前余额	可用余额	查询时间	操作
        4301016509812100729	英镑	汇	基本账户	600,000.00	600,000.00	600,000.00	15:50:08	
        4301016509813100767	港币	汇	基本账户	20,004,076.60	20,004,076.60	20,004,076.60	15:50:08'''

    # 文件开始取数索引
    ROW_START_IDX = 2

    def __init__(self, file_path, row_start_idx=ROW_START_IDX):
        super().__init__(file_path, row_start_idx)

    def build_self(self, row_idx):
        sheet = self.excel_sheet
        # 依次按列取值
        self.acc_no = sheet.cell(row_idx, 0).value
        self.acc_name = CONSTANTS.BANK_NO_DICT[self.acc_no]
        self.balance = sheet.cell(row_idx, 6).value
        self.query_date = sheet.cell(row_idx, 7).value
        self.currency = sheet.cell(row_idx, 1).value
        return self


class HXBBalance(BaseBalance):

    ''' 工商银行
        序号	账户别名	账号	账户名称	币种	可用余额
        1	未定义	12592000000000515	江苏银行股份有限公司无锡分行	人民币	10,211.33'''

    # 文件开始取数索引
    ROW_START_IDX = 1

    def __init__(self, file_path, row_start_idx=ROW_START_IDX):
        super().__init__(file_path, row_start_idx)

    def build_self(self, row_idx):
        sheet = self.excel_sheet
        # 依次按列取值
        self.acc_no = sheet.cell(row_idx, 2).value
        self.acc_name = CONSTANTS.BANK_NO_DICT[self.acc_no]
        self.balance = sheet.cell(row_idx, 5).value
        self.query_date = ""
        self.currency = sheet.cell(row_idx, 4).value
        return self


if (__name__ == "__main__"):
    bfr = HXBBalance(
        r"D:\Repos\华夏银行\20230606\华夏银行无锡分行_20230606_余额.xlsx").to_sqlite()




BANK_TRANS_CLS = {"工商银行": "ICBCTrans",
                  "华夏银行": "HXBTrans"}


BANK_NAME_IDX = 2
BANK_NO_IDX = -1
SQLITE3_PATH = r"D:\rpadbs\ebank.db"


BANK_NO_DICT = {
    "4301016509812100729": "江苏银行股份有限公司",
    "4301016509813100767": "江苏银行股份有限公司",
    "4301016509814104041": "江苏银行股份有限公司",
    "4301016509827101946": "江苏银行股份有限公司",
    "4301016509829100872": "江苏银行股份有限公司",
    "4301016509838105663": "江苏银行股份有限公司",
    "4301016519100210521": "江苏银行股份有限公司",
    "4301016529100527482": "江苏银行股份有限公司",
    "12592000000000515": "江苏银行股份有限公司无锡分行"
}

SQL_INSERT_TRANS = 'insert into trans(file_path,bank_name,acc_name,acc_no,trans_date,flow_no,rec_amt,pay_amt,' + \
    ' balance,other_acc_name,other_acc_no,other_bank_name,purpose,abstract,remark,rct_path, task_id,' + \
    ' create_date,update_date,status) ' + \
    ' values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)'

SQL_QUERY_TRANS_BYDATE = 'SELECT * FROM trans WHERE trans_date BETWEEN \'{0}\' AND \'{1}\''

SQL_INSERT_BALANCE = 'insert into balance(file_path,bank_name,acc_name,acc_no,' + \
    ' balance, query_date,currency,task_id,' + \
    ' create_date,update_date,status) ' + \
    ' values(?,?,?,?,?,?,?,?,?,?,?)'



from setuptools import find_packages, setup

setup(
    name='ebank',
    version='2023.6.8.4',
    packages=find_packages(),
    install_requires=['xlwt>=1.3.0', 'xlrd==1.2.0',
                      'pdfplumber>=0.8.0', "encoo==2023.6.8",
                      "python-dateutil>=2.8.2", "pypdf>=3.6.0", "requests>=2.28.2", "urllib3>=1.26.14"]
)


# 打包 python .\setup.py sdist bdist_wheel
# 上传 twine upload dist/*
# 安装 pip install ebanktool==0.0.1  -i https://www.pypi.org/simple/
